#include <netdb.h>
#include <netinet/in.h>
#include <readline/history.h>
#include <readline/readline.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/un.h>
#include <termios.h>
#include <unistd.h>

#include <cassert>
#include <iostream>
#include <memory>
#include <string>
#include <chrono>
#include <fstream>
#include <sstream>
#include <vector>
#define MAX_MEM_BUFFER_SIZE 8192
#define PORT_DEFAULT 8765

bool is_exit_command(std::string &cmd) { return cmd == "exit" || cmd == "exit;" || cmd == "bye" || cmd == "bye;"; }

int init_unix_sock(const char *unix_sock_path) {
    int sockfd = socket(PF_UNIX, SOCK_STREAM, 0);
    if (sockfd < 0) {
        fprintf(stderr, "failed to create unix socket. %s", strerror(errno));
        return -1;
    }

    struct sockaddr_un sockaddr;
    memset(&sockaddr, 0, sizeof(sockaddr));
    sockaddr.sun_family = PF_UNIX;
    snprintf(sockaddr.sun_path, sizeof(sockaddr.sun_path), "%s", unix_sock_path);

    if (connect(sockfd, (struct sockaddr *)&sockaddr, sizeof(sockaddr)) < 0) {
        fprintf(stderr, "failed to connect to server. unix socket path '%s'. error %s", sockaddr.sun_path,
                strerror(errno));
        close(sockfd);
        return -1;
    }
    return sockfd;
}

int init_tcp_sock(const char *server_host, int server_port) {
    struct hostent *host;
    struct sockaddr_in serv_addr;

    if ((host = gethostbyname(server_host)) == NULL) {
        fprintf(stderr, "gethostbyname failed. errmsg=%d:%s\n", errno, strerror(errno));
        return -1;
    }

    int sockfd;
    if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
        fprintf(stderr, "create socket error. errmsg=%d:%s\n", errno, strerror(errno));
        return -1;
    }

    serv_addr.sin_family = AF_INET;
    serv_addr.sin_port = htons(server_port);
    serv_addr.sin_addr = *((struct in_addr *)host->h_addr);
    bzero(&(serv_addr.sin_zero), 8);

    if (connect(sockfd, (struct sockaddr *)&serv_addr, sizeof(struct sockaddr)) == -1) {
        fprintf(stderr, "Failed to connect. errmsg=%d:%s\n", errno, strerror(errno));
        close(sockfd);
        return -1;
    }
    return sockfd;
}

void exec_command(int sockfd, const std::string &command) {
    int send_bytes;
    char recv_buf[MAX_MEM_BUFFER_SIZE];

    std::cout << "Executing: " << command << std::endl;
    if ((send_bytes = write(sockfd, command.c_str(), command.length() + 1)) == -1) {
        std::cerr << "send error: " << errno << ":" << strerror(errno) << " \n" << std::endl;
        exit(1);
    }
    int len = recv(sockfd, recv_buf, MAX_MEM_BUFFER_SIZE, 0);
    if (len < 0) {
        fprintf(stderr, "Connection was broken: %s\n", strerror(errno));
        exit(1);
    } else if (len == 0) {
        printf("Connection has been closed\n");
        exit(1);
    } else {
        for (int i = 0; i <= len; i++) {
            if (recv_buf[i] == '\0') {
                break;
            } else {
                printf("%c", recv_buf[i]);
            }
        }
        printf("\n");
    }
    memset(recv_buf, 0, MAX_MEM_BUFFER_SIZE);
}

void exec_file_commands(int sockfd, const std::string &filename) {
    std::ifstream file(filename);
    if (!file.is_open()) {
        std::cerr << "Failed to open file: " << filename << std::endl;
        return;
    }

    std::string line;
    while (std::getline(file, line)) {
        std::stringstream ss(line);
        std::string command;
        while (std::getline(ss, command, ';')) {
            if (!command.empty()) {
                exec_command(sockfd, command + ";");
            }
        }
    }
}

void generate_sql_file(const std::string &filename) {
    if (std::ifstream(filename)) {
        if (remove(filename.c_str()) != 0) {
            std::cerr << "Failed to delete existing file." << std::endl;
            return;
        }
    }

    std::ofstream outfile(filename);
    if (!outfile.is_open()) {
        std::cerr << "Failed to open file for writing." << std::endl;
        return;
    }
    // Add CREATE TABLE statement
    outfile << "create table student (id int, name char(32), score float);\n";
//    outfile << "create index student (id);\n";
//    outfile << "create index student (id,name);\n";
    outfile << "begin;\n";
    for (int i = 1; i < 100; i = i  + 2) {
        std::ostringstream ss;
        ss << "insert into student values ("
           << i << ", 'student" << i << "', " << (90.0 + i % 10) << ");\n";
        outfile << ss.str();
    }
    outfile << "commit;\n";
    outfile << "select * from student;\n";
    outfile << "begin;\n";
    for (int i = 2; i < 100; i = i + 2) {
        std::ostringstream ss;
        ss << "update student set id = " << i * (-1) << " where id = " << i << ";\n";
        ss << "delete from student where id = " << i <<";\n";
        ss << "insert into student values ("
           << i << ", 'student" << i << "', " << (90.0 + i % 10) << ");\n";
        outfile << ss.str();
    }
    outfile << ";";
    outfile.close();
}

void generate_count_sql_file(const std::string &filename) {
    if (std::ifstream(filename)) {
        if (remove(filename.c_str()) != 0) {
            std::cerr << "Failed to delete existing file." << std::endl;
            return;
        }
    }

    std::ofstream outfile(filename);
    if (!outfile.is_open()) {
        std::cerr << "Failed to open file for writing." << std::endl;
        return;
    }

    for (int w_id = 1; w_id <= 21; ++w_id) {
        for (int d_id = 1; d_id <= 11; ++d_id) {
            outfile << "select COUNT(ol_o_id) as count_ol_o_id from order_line where ol_w_id = " << w_id << " and ol_d_id = " << d_id << ";\n";
            outfile << "select COUNT(no_o_id) as count_no_o_id from new_orders where no_w_id = " << w_id << " and no_d_id = " << d_id << ";\n";
        }
    }
    outfile << ";";
    outfile.close();
}
void generate_max_sql_file(const std::string &filename) {
    if (std::ifstream(filename)) {
        if (remove(filename.c_str()) != 0) {
            std::cerr << "Failed to delete existing file." << std::endl;
            return;
        }
    }

    std::ofstream outfile(filename);
    if (!outfile.is_open()) {
        std::cerr << "Failed to open file for writing." << std::endl;
        return;
    }

    for (int w_id = 1; w_id <= 21; ++w_id) {
        for (int d_id = 1; d_id <= 11; ++d_id) {
            outfile << "select MAX(ol_o_id) as count_ol_o_id from order_line where ol_w_id = " << w_id << " and ol_d_id = " << d_id << ";\n";
            outfile << "select MAX(no_o_id) as count_no_o_id from new_orders where no_w_id = " << w_id << " and no_d_id = " << d_id << ";\n";
        }
    }
    outfile << ";";
    outfile.close();
}

void generate_min_sql_file(const std::string &filename) {
    if (std::ifstream(filename)) {
        if (remove(filename.c_str()) != 0) {
            std::cerr << "Failed to delete existing file." << std::endl;
            return;
        }
    }

    std::ofstream outfile(filename);
    if (!outfile.is_open()) {
        std::cerr << "Failed to open file for writing." << std::endl;
        return;
    }

    for (int w_id = 1; w_id <= 21; ++w_id) {
        for (int d_id = 1; d_id <= 15; ++d_id) {
            outfile << "select MIN(ol_o_id) as count_ol_o_id from order_line where ol_w_id = " << w_id << " and ol_d_id = " << d_id << ";\n";
            outfile << "select MIN(no_o_id) as count_no_o_id from new_orders where no_w_id = " << w_id << " and no_d_id = " << d_id << ";\n";
        }
    }
    outfile << ";";
    outfile.close();
}
void generate_sum_sql_file(const std::string &filename) {
    if (std::ifstream(filename)) {
        if (remove(filename.c_str()) != 0) {
            std::cerr << "Failed to delete existing file." << std::endl;
            return;
        }
    }

    std::ofstream outfile(filename);
    if (!outfile.is_open()) {
        std::cerr << "Failed to open file for writing." << std::endl;
        return;
    }

    for (int w_id = 1; w_id <= 25; ++w_id) {
        for (int d_id = 1; d_id <= 15; ++d_id) {
            outfile << "select SUM(ol_o_id) as count_ol_o_id from order_line where ol_w_id = " << w_id << " and ol_d_id = " << d_id << ";\n";
            outfile << "select SUM(no_o_id) as count_no_o_id from new_orders where no_w_id = " << w_id << " and no_d_id = " << d_id << ";\n";
        }
    }
    outfile << ";";
    outfile.close();
}


int main(int argc, char *argv[]) {
    int ret = 0;  // set_terminal_noncanonical();
                  //    if (ret < 0) {
                  //        printf("Warning: failed to set terminal non canonical. Long command may be "
                  //               "handled incorrect\n");
                  //    }

    const char *unix_socket_path = nullptr;
    const char *server_host = "127.0.0.1";  // 127.0.0.1 192.168.31.25
    const char *file_path = nullptr;
    int server_port = PORT_DEFAULT;
    int opt;

    while ((opt = getopt(argc, argv, "s:h:p:f:")) > 0) {
        switch (opt) {
            case 's':
                unix_socket_path = optarg;
                break;
            case 'p':
                char *ptr;
                server_port = (int)strtol(optarg, &ptr, 10);
                break;
            case 'h':
                server_host = optarg;
                break;
            case 'f':
                file_path = optarg;
            default:
                break;
        }
    }

    // const char *prompt_str = "RucBase > ";

    int sockfd, send_bytes;
    // char send[MAXLINE];

    if (unix_socket_path != nullptr) {
        sockfd = init_unix_sock(unix_socket_path);
    } else {
        sockfd = init_tcp_sock(server_host, server_port);
    }
    if (sockfd < 0) {
        return 1;
    }

    char recv_buf[MAX_MEM_BUFFER_SIZE];

//
//    std::string creat_sql = "/home/ll/Desktop/db2024-x/rmdb_client/test/create_table.sql";
//    std::string load_sql = "/home/ll/Desktop/db2024-x/rmdb_client/test/load.sql";
//    std::string consistency_sql = "/home/ll/Desktop/db2024-x/rmdb_client/test/test.sql";
//    std::string count_sql = "/home/ll/Desktop/db2024-x/rmdb_client/test/count.sql";
//    std::string min_sql = "/home/ll/Desktop/db2024-x/rmdb_client/test/min.sql";
//    std::string max_sql = "/home/ll/Desktop/db2024-x/rmdb_client/test/max.sql";
//    std::string sum_sql = "/home/ll/Desktop/db2024-x/rmdb_client/test/sum.sql";
//    generate_count_sql_file(count_sql);
//    generate_count_sql_file(min_sql);
//    generate_count_sql_file(max_sql);
//    generate_count_sql_file(sum_sql);
//
//    exec_file_commands(sockfd,creat_sql);
//
//    auto load_start = std::chrono::high_resolution_clock::now();
//    exec_file_commands(sockfd,load_sql);
//    auto load_end = std::chrono::high_resolution_clock::now();
//    std::chrono::duration<double> duration = std::chrono::duration_cast<std::chrono::duration<double>>(load_end - load_start);
//    std::cout << "Time taken to execute load.sql: " << duration.count() << " seconds." << std::endl;
//
//    auto test_sql_start = std::chrono::high_resolution_clock::now();
//    exec_file_commands(sockfd,consistency_sql);
//    exec_file_commands(sockfd,count_sql);
//    exec_file_commands(sockfd,min_sql);
//    exec_file_commands(sockfd,max_sql);
//    exec_file_commands(sockfd,sum_sql);
//    auto test_sql_end = std::chrono::high_resolution_clock::now();
//    std::chrono::duration<double> duration2 = std::chrono::duration_cast<std::chrono::duration<double>>(test_sql_end - test_sql_start);
//    std::cout << "Time taken to execute consistency.sql: " << duration2.count() << " seconds." << std::endl;
//



//    std::string sql_file = "insert_commands.sql";
//    generate_sql_file(sql_file);
//    exec_file_commands(sockfd, sql_file);


//    if (file_path != nullptr) {
//        exec_file_commands(sockfd, file_path);
//    } else {
//        exec_file_commands(sockfd, sql_file);
//    }
    while (1) {
        char *line_read = readline("Rucbase> ");
        if (line_read == nullptr) {
            // EOF encountered
            break;
        }
        std::string command = line_read;
        free(line_read);

        if (!command.empty()) {
            add_history(command.c_str());
            if (is_exit_command(command)) {
                printf("The client will be closed.\n");
                break;
            }

            if ((send_bytes = write(sockfd, command.c_str(), command.length() + 1)) == -1) {
                // fprintf(stderr, "send error: %d:%s \n", errno, strerror(errno));
                std::cerr << "send error: " << errno << ":" << strerror(errno) << " \n" << std::endl;
                exit(1);
            }
            int len = recv(sockfd, recv_buf, MAX_MEM_BUFFER_SIZE, 0);
            if (len < 0) {
                fprintf(stderr, "Connection was broken: %s\n", strerror(errno));
                break;
            } else if (len == 0) {
                printf("Connection has been closed\n");
                break;
            } else {
                for (int i = 0; i <= len; i++) {
                    if (recv_buf[i] == '\0') {
                        break;
                    } else {
                        printf("%c", recv_buf[i]);
                    }
                }
                memset(recv_buf, 0, MAX_MEM_BUFFER_SIZE);
            }
        }
    }
    close(sockfd);
    printf("Bye.\n");
    return 0;
}
