#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <random>
#include <ctime>
#include <iomanip>
#include <unistd.h>
// Random number generator setup
std::random_device rd;
std::mt19937 gen(rd());

std::string random_string(size_t length) {
    const char charset[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
    std::uniform_int_distribution<> dist(0, sizeof(charset) - 2);
    std::string result;
    for (size_t i = 0; i < length; ++i) {
        result += charset[dist(gen)];
    }
    return result;
}

std::string random_date() {
    std::time_t t = std::time(0) + std::uniform_int_distribution<>(0, 365 * 24 * 3600)(gen);
    std::tm *now = std::localtime(&t);
    std::ostringstream oss;
    oss << std::put_time(now, "%Y-%m-%d %H:%M:%S");
    return oss.str();
}
void generate_warehouse(const std::string& prefix,int W) {
    std::string filename = prefix + "warehouse.csv";
    std::ofstream file(filename);
    file << "w_id,w_name,w_street_1,w_street_2,w_city,w_state,w_zip,w_tax,w_ytd\n";
    for (int i = 1; i <= W; ++i) {
        file << i << "," << random_string(10) << "," << random_string(15) << "," << random_string(15) << ","
             << random_string(10) << "," << random_string(2) << "," << random_string(8) << ","
             << std::uniform_real_distribution<>(0.0, 1.0)(gen) << "," << std::uniform_real_distribution<>(1000.0, 5000.0)(gen) << "\n";
    }
    file.close();

    // 获取绝对路径并输出
    char absolute_path[1000];
    if (realpath(filename.c_str(), absolute_path) != nullptr) {
        std::cout << "File saved at: " << absolute_path << std::endl;
    } else {
        std::cerr << "Failed to resolve absolute path" << std::endl;
    }
}

void generate_item(const std::string& prefix) {
    std::ofstream file(prefix + "item.csv");
    file << "i_id,i_im_id,i_name,i_price,i_data\n";
    for (int i = 1; i <= 100000; ++i) {
        file << i << "," << std::uniform_int_distribution<>(1, 5000)(gen) << "," << random_string(20) << ","
             << std::uniform_real_distribution<>(10.0, 1000.0)(gen) << "," << random_string(30) << "\n";
    }
    file.close();
}

void generate_stock(const std::string& prefix,int W) {
    std::ofstream file(prefix + "stock.csv");
    file << "s_i_id,s_w_id,s_quantity,s_dist_01,s_dist_02,s_dist_03,s_dist_04,s_dist_05,s_dist_06,s_dist_07,s_dist_08,s_dist_09,s_dist_10,s_ytd,s_order_cnt,s_remote_cnt,s_data\n";
    for (int w = 1; w <= W; ++w) {
        for (int i = 1; i <= 100000; ++i) {
            file << i << "," << w << "," << std::uniform_int_distribution<>(10, 100)(gen) << ","
                 << random_string(24) << "," << random_string(24) << "," << random_string(24) << ","
                 << random_string(24) << "," << random_string(24) << "," << random_string(24) << ","
                 << random_string(24) << "," << random_string(24) << "," << random_string(24) << ","
                 << random_string(24) << "," << std::uniform_real_distribution<>(0.0, 5000.0)(gen) << ","
                 << std::uniform_int_distribution<>(0, 100)(gen) << "," << std::uniform_int_distribution<>(0, 100)(gen) << ","
                 << random_string(50) << "\n";
        }
    }
    file.close();
}

void generate_district(const std::string& prefix,int W) {
    std::ofstream file(prefix + "district.csv");
    file << "d_id,d_w_id,d_name,d_street_1,d_street_2,d_city,d_state,d_zip,d_tax,d_ytd,d_next_o_id\n";
    for (int w = 1; w <= W; ++w) {
        for (int d = 1; d <= 10; ++d) {
            file << d << "," << w << "," << random_string(10) << "," << random_string(15) << ","
                 << random_string(15) << "," << random_string(10) << "," << random_string(2) << ","
                 << random_string(8) << "," << std::uniform_real_distribution<>(0.0, 1.0)(gen) << ","
                 << std::uniform_real_distribution<>(10000.0, 30000.0)(gen) << ","
                 << std::uniform_int_distribution<>(1, 3001)(gen) << "\n";
        }
    }
    file.close();
}

void generate_customer(const std::string& prefix,int W) {
    std::ofstream file(prefix + "customer.csv");
    file << "c_id,c_d_id,c_w_id,c_first,c_middle,c_last,c_street_1,c_street_2,c_city,c_state,c_zip,c_phone,c_since,c_credit,c_credit_lim,c_discount,c_balance,c_ytd_payment,c_payment_cnt,c_delivery_cnt,c_data\n";
    for (int w = 1; w <= W; ++w) {
        for (int d = 1; d <= 10; ++d) {
            for (int c = 1; c <= 3000; ++c) {
                file << c << "," << d << "," << w << "," << random_string(10) << "," << random_string(2) << ","
                     << random_string(10) << "," << random_string(20) << "," << random_string(20) << ","
                     << random_string(10) << "," << random_string(2) << "," << random_string(8) << ","
                     << random_string(16) << "," << random_date() << "," << (rand() % 2 ? "GC" : "BC") << ","
                     << std::uniform_real_distribution<>(1000.0, 50000.0)(gen) << "," << std::uniform_real_distribution<>(0.0, 10.0)(gen) << ","
                     << std::uniform_real_distribution<>(0.0, 5000.0)(gen) << "," << std::uniform_real_distribution<>(0.0, 5000.0)(gen) << ","
                     << std::uniform_int_distribution<>(0, 10)(gen) << "," << std::uniform_int_distribution<>(0, 10)(gen) << ","
                     << random_string(50) << "\n";
            }
        }
    }
    file.close();
}

void generate_history(const std::string& prefix ,int W) {
    std::ofstream file(prefix + "history.csv");
    file << "h_c_id,h_c_d_id,h_c_w_id,h_d_id,h_w_id,h_date,h_amount,h_data\n";
    for (int w = 1; w <= W; ++w) {
        for (int d = 1; d <= 10; ++d) {
            for (int c = 1; c <= 3000; ++c) {
                file << c << "," << d << "," << w << "," << d << "," << w << "," << random_date() << ","
                     << std::uniform_real_distribution<>(0.0, 5000.0)(gen) << "," << random_string(50) << "\n";
            }
        }
    }
    file.close();
}

void generate_orders(const std::string& prefix,int W) {
    std::ofstream file(prefix + "orders.csv");
    file << "o_id,o_d_id,o_w_id,o_c_id,o_entry_d,o_carrier_id,o_ol_cnt,o_all_local\n";
    for (int w = 1; w <= W; ++w) {
        for (int d = 1; d <= 10; ++d) {
            for (int o = 1; o <= 3000; ++o) {
                file << o << "," << d << "," << w << "," << std::uniform_int_distribution<>(1, 3000)(gen) << ","
                     << random_date() << "," << std::uniform_int_distribution<>(1, 10)(gen) << ","
                     << std::uniform_int_distribution<>(5, 15)(gen) << "," << 1 << "\n";
            }
        }
    }
    file.close();
}

void generate_new_orders(const std::string& prefix,int W) {
    std::ofstream file(prefix + "new_orders.csv");
    file << "no_o_id,no_d_id,no_w_id\n";
    for (int w = 1; w <= W; ++w) {
        for (int d = 1; d <= 10; ++d) {
            for (int no = 2101; no <= 3000; ++no) {
                file << no << "," << d << "," << w << "\n";
            }
        }
    }
    file.close();
}

void generate_order_line(const std::string& prefix,int W) {
    std::ofstream file(prefix + "order_line.csv");
    file << "ol_o_id,ol_d_id,ol_w_id,ol_number,ol_i_id,ol_supply_w_id,ol_delivery_d,ol_quantity,ol_amount,ol_dist_info\n";
    for (int w = 1; w <= W; ++w) {
        for (int d = 1; d <= 10; ++d) {
            for (int o = 1; o <= 3000; ++o) {
                int ol_cnt = std::uniform_int_distribution<>(5, 15)(gen);
                for (int ol = 1; ol <= ol_cnt; ++ol) {
                    file << o << "," << d << "," << w << "," << ol << ","
                         << std::uniform_int_distribution<>(1, 100000)(gen) << "," << w << "," << random_date() << ","
                         << std::uniform_int_distribution<>(1, 10)(gen) << "," << std::uniform_real_distribution<>(0.0, 100.0)(gen) << ","
                         << random_string(24) << "\n";
                }
            }
        }
    }
    file.close();
}

int main() {
    int W = 20; // Example W value, can be set to any value up to 50
///home/ll/Desktop/db2024-x/customer.csv
    ///home/ll/Desktop/db2024-x/rmdb_client/test/main.cpp

//    std::string prefix = "../../src/test/performance_test/table_data/";
    std::string prefix = "/home/ll/Desktop/db2024-x/src/test/performance_test/table_data/";
    generate_warehouse(prefix,W);
    std::cout<<"warehoue_ok"<<std::endl;
    generate_item(prefix);
    generate_stock(prefix,W);
    std::cout<<"stock_ok"<<std::endl;
    generate_district(prefix,W);
    generate_customer(prefix,W);
    generate_history(prefix,W);
    generate_orders(prefix,W);
    generate_new_orders(prefix,W);
    generate_order_line(prefix,W);

    std::cout << "Data generation completed." << std::endl;
    return 0;
}